@@ -170,33 +170,90 @@ sr_vcov <- function(X,vcov.func=vcov,ope=1) {
 # See Walck, section 33.5
 .t_se_normal <- function(tstat,df) {
 	se <- sqrt(1 + (tstat**2) / (2*df))
-	return(se)
 }
-.t_se <- function(t,df,type=c("t","Lo")) {
-	# 2FIX: add opdyke corrections for skew and kurtosis?
+# note that typically the df given here is n-1?
+# just sweep that under the rug.
+.t_se_Mertens <- function(tstat,df,cumulants) {
+	stopifnot(!is.null(cumulants))
+	# hey, how is this supposed to work with matrices??
+	if (!is.matrix(cumulants)) { cumulants <- matrix(cumulants,ncol=1) }
+	se <- sqrt(1 - (cumulants[1,,drop=TRUE] * tstat / sqrt(df+1)) + (cumulants[2,,drop=TRUE] + 2) * (tstat**2) / (4*(df+1)))
+} 
+.t_se_Bao <- function(tstat,df,cumulants) {
+	stopifnot(!is.null(cumulants))
+	if (!is.matrix(cumulants)) { cumulants <- matrix(cumulants,ncol=1) }
+	# this is from Yong Bao's code:
+	S <- tstat / sqrt(df+1)
+	se <- sqrt((df+1) * ( (1+S^2/2)/(df+1)+(19*S^2/8+2)/(df+1)^2-cumulants[1,,drop=TRUE]*S*(1/(df+1)+5/2/(df+1)^2)
+    +cumulants[2,,drop=TRUE]*S^2*(1/4/(df+1)+3/8/(df+1)^2)+5*cumulants[3,,drop=TRUE]*S/4/(df+1)^2-3*cumulants[4,,drop=TRUE]*S^2/8/(df+1)^2
+    +cumulants[1,,drop=TRUE]^2*(7/4/(df+1)^2-3*S^2/2/(df+1)^2)
+    -15*cumulants[1,,drop=TRUE]*cumulants[2,,drop=TRUE]*S/4/(df+1)^2+39*cumulants[2,,drop=TRUE]^2*S^2/32/(df+1)^2 ));
+} 
+
+ 
+# note that the cumulants are assumed on the returns distribution,
+# not the cumulants of the t-stat, of course.
+.t_se <- function(t,df,type=c("t","Lo","Mertens","Bao"),cumulants=NULL) {
 	# 2FIX: add autocorrelation correction?
 	type <- match.arg(type)
-	se <- switch(type,
-							 t = .t_se_normal(t,df),
-							 Lo = .t_se_normal(t,df))
-	return(se)
+	switch(type,
+				 t = .t_se_normal(t,df),
+				 Lo = .t_se_normal(t,df),
+				 Mertens = .t_se_Mertens(t,df,cumulants),
+				 Bao = .t_se_Bao(t,df,cumulants))
 }
+
+#.t_bias1 <- function(tstat,df,cumulants) {
+	#if (!is.matrix(cumulants)) { cumulants <- matrix(cumulants,ncol=1) }
+	#(2 + cumulants[2,,drop=TRUE])*tstat*3/8/(df+1)^(3/2) - cumulants[1,,drop=TRUE]/2/(df+1);
+#}
+.t_bias2 <- function(tstat,df,cumulants) {
+	if (!is.matrix(cumulants)) { cumulants <- matrix(cumulants,ncol=1) }
+	3*tstat/4/(df+1)^(3/2) +
+		3/8/(df+1)^2 * (cumulants[3,,drop=TRUE] - 5*cumulants[1,,drop=TRUE]*cumulants[2,,drop=TRUE]/2) + 
+		cumulants[1,,drop=TRUE]*(1/2/(df+1)+3/8/(df+1)^2) +
+		tstat*cumulants[2,,drop=TRUE]*(3/8 -15/32/(df+1))/(df+1)^(3/2) +
+		5*tstat/16/(df+1)^(5/2) * (49/10 - cumulants[4,,drop=TRUE] - 4 * cumulants[1,,drop=TRUE]^2 + 21*cumulants[2,,drop=TRUE]^2/8) 
+}
+
+# recentering to remove bias
+.t_recenter <- function(tstat,df,type=c("t","Lo","Z","Mertens","Bao"),cumulants=NULL) {
+	type <- match.arg(type)
+	if (type %in% c("t","Lo","Mertens")) { return(tstat) }
+	retv <- switch(type,
+								 Z={
+									 tstat * (1 - 1 / (4 * df))
+								 },
+								 Bao={
+									 tstat - .t_bias2(tstat,df,cumulants=cumulants)
+								 })
+}
+
 # confidence intervals on the non-centrality parameter of a t-stat
-.t_confint <- function(tstat,df,level=0.95,type=c("exact","t","Z"),
-					 level.lo=(1-level)/2,level.hi=1-level.lo) {
+.t_confint <- function(tstat,df,level=0.95,type=c("exact","t","Z","Mertens","Bao"),
+					 level.lo=(1-level)/2,level.hi=1-level.lo,cumulants=cumulants) {
 	type <- match.arg(type)
 	if (type == "exact") {
 		ci.lo <- qlambdap(level.lo,df,tstat,lower.tail=TRUE)
 		ci.hi <- qlambdap(level.hi,df,tstat,lower.tail=TRUE)
 		ci <- cbind(ci.lo,ci.hi)
 	} else {
-		if (type == "t") {
-			se <- .t_se(tstat,df,type=type)
-			midp <- tstat
-		} else if (type == "Z") {
-			se <- .t_se(tstat,df,type="t")
-			midp <- tstat * (1 - 1 / (4 * df))
-		} else stop("internal error")
+		midp <- tstat  # mostly we use the tstat as the middle point, unless bias correcting.
+		switch(type,
+					 t={
+						 se <- .t_se(tstat,df,type="t")
+					 },
+					 Z={  # this is odd: we unbias the SR based on the simple bias correction
+						 se <- .t_se(tstat,df,type="t")
+						 midp <- .t_recenter(tstat,df,type=type)
+					 },
+					 Mertens={
+						 se <- .t_se(tstat,df,type="Mertens",cumulants=cumulants)
+					 },
+					 Bao={
+						 se <- .t_se(tstat,df,type="Bao",cumulants=cumulants)
+						 midp <- .t_recenter(tstat,df,type=type,cumulants=cumulants)
+					 })
 		zalp <- qnorm(c(level.lo,level.hi))
 		ci <- cbind(midp + zalp[1] * se,midp + zalp[2] * se)
 	} 
@@ -223,31 +280,36 @@ se <- function(z, type) {
 #' @details 
 #'
 #' For an observed Sharpe ratio, estimate the standard error.
-#' There are two methods:
+#' The following methods are recognized:
 #'
-#' \itemize{
-#' \item The default, \code{t}, based on Johnson & Welch, with a correction
-#' for small sample size, also known as \code{Lo}.
-#' \item A method based on the exact variance of the non-central t-distribution,
-#' \code{exact}.
+#' \describe{
+#' \item{t}{The default, based on Johnson & Welch, with a correction
+#' for small sample size. Also known as \code{'Lo'}.}
+#' \item{Mertens}{An approximation to the standard error taking into
+#' skewness and kurtosis of the returns distribution.}
+#' \item{Bao}{An even higher accuracty approximation using higher order
+#' moments.}
 #' }
+#'
 #' There should be very little difference between these except for very small
 #' sample sizes.
 #'
 #' @param z an observed Sharpe ratio statistic, of class \code{sr}.
-#' @param type estimator type. one of \code{"t", "Lo", "exact"}
+#' @param type estimator type. one of \code{"t", "Lo", "Mertens", "Bao"}
 #' @template param-ellipsis
 #' @keywords htest
 #' @return an estimate of standard error.
-#' @seealso sr-distribution functions, \code{\link{dsr}}
+#' @seealso sr-distribution functions, \code{\link{dsr}},
+#' \code{\link{sr_variance}}.
 #' @export 
 #' @template etc
 #' @template sr
 #' @note
-#' Eventually this should include corrections for autocorrelation, skew,
-#' kurtosis.
+#' The units of the standard error are consistent with those of the
+#' input \code{sr} object.
 #' @template ref-JW
 #' @template ref-Lo
+#' @template ref-Bao
 #' @template ref-Opdyke
 #' @references 
 #'
@@ -260,12 +322,11 @@ se <- function(z, type) {
 #' anse <- se(asr,type="Lo")
 #'
 #' @method se sr
-#' @S3method se sr
 #' @rdname se
 #' @export
-se.sr <- function(z, type=c("t","Lo")) {
+se.sr <- function(z, type=c("t","Lo","Mertens","Bao")) {
 	tstat <- .sr2t(z)
-	retval <- .t_se(tstat,df=z$df,type=type)
+	retval <- .t_se(tstat,df=z$df,type=type,cumulants=z$cumulants)
 	retval <- .t2sr(z,retval)
 	return(retval)
 }
@@ -273,16 +334,6 @@ se.sr <- function(z, type=c("t","Lo")) {
 
 # confidence intervals on the Sharpe ratio#FOLDUP
 
-# do not have to include this, as confint is a generic provided
-# by R. I think.
-#  ' @usage
-#  '
-#  ' confint(object,level=0.95,level.lo=(1-level)/2,level.hi=1-level.lo,...)
-#confint <- function(object,level=0.95,
-							 #level.lo=(1-level)/2,level.hi=1-level.lo,...) {
-	#UseMethod("confint", object)
-#}
-
 #' @title Confidence Interval on (optimal) Signal-Noise Ratio
 #'
 #' @description 
@@ -296,12 +347,18 @@ se.sr <- function(z, type=c("t","Lo")) {
 #' Constructs confidence intervals on the Signal-Noise ratio given observed
 #' Sharpe ratio statistic. The available methods are:
 #'
-#' \itemize{
-#' \item The default, \code{exact}, which is only exact when returns are
-#' normal, based on inverting the non-central t
-#' distribution.
-#' \item A method based on the standard error of a non-central t distribution.
-#' \item A method based on a normal approximation.
+#' \describe{
+#' \item{exact}{The default, which is only exact when returns are
+#' normal, based on inverting the non-central t distribution.}
+#' \item{t}{Uses the Johnson Welch approximation to the standard error, centered around
+#' the sample value.}
+#' \item{Z}{Uses the Johnson Welch approximation to the standard error,
+#' performing a simple correction for the bias of the Sharpe ratio based on 
+#' Miller and Gehr formula.}
+#' \item{Mertens}{Uses the Mertens higher order approximation to the standard
+#' error, centered around the sample value.}
+#' \item{Bao}{Uses the Bao higher order approximation to the standard error,
+#' performing a higher order correction for the bias of the Sharpe ratio.}
 #' }
 #'
 #' Suppose \eqn{x_i}{xi} are \eqn{n}{n} independent draws of a \eqn{q}{q}-variate
@@ -382,16 +439,15 @@ se.sr <- function(z, type=c("t","Lo")) {
 #' aci <- confint(asro,level=0.95)
 #'
 #' @method confint sr 
-#' @S3method confint sr 
 #' @export
 confint.sr <- function(object,parm,level=0.95,
 							 level.lo=(1-level)/2,level.hi=1-level.lo,
-							 type=c("exact","t","Z"),...) {
+							 type=c("exact","t","Z","Mertens","Bao"),...) {
 	type <- match.arg(type)
 	tstat <- .sr2t(object)
 	retval <- .t_confint(tstat,df=object$df,level=level,
 											 level.lo=level.lo,level.hi=level.hi,
-											 type=type)
+											 type=type,cumulants=object$cumulants)
 	retval <- .t2sr(object,retval)
 	rownames(retval) <- .get_strat_names(object$sr)
 	return(retval)
@@ -399,7 +455,6 @@ confint.sr <- function(object,parm,level=0.95,
 #' @export
 #' @rdname confint
 #' @method confint sropt
-#' @S3method confint sropt
 confint.sropt <- function(object,parm,level=0.95,
 							 level.lo=(1-level)/2,level.hi=1-level.lo,...) {
 	ci.hi <- qco_sropt(level.hi,df1=object$df1,df2=object$df2,
@@ -415,7 +470,6 @@ confint.sropt <- function(object,parm,level=0.95,
 #' @export
 #' @rdname confint
 #' @method confint del_sropt
-#' @S3method confint del_sropt
 confint.del_sropt <- function(object,parm,level=0.95,
 							 level.lo=(1-level)/2,level.hi=1-level.lo,...) {
 	Fandp <- .del_sropt.asF(object)
@@ -554,7 +608,7 @@ predint <- function(x,oosdf,oosrescal=1/sqrt(oosdf+1),ope=NULL,level=0.95,
 }
 
 #MLE of the ncp based on a single F-stat
-.F_ncp_MLE_single <- function(Fs,df1,df2,ub=NULL,lb=0) { # nocov start
+.F_ncp_MLE_single <- function(Fs,df1,df2,ub=NULL,lb=0) { 
 	if (Fs <= 1) { return(0.0) }  # Spruill's Thm 3.1, eqn 8
 	max.func <- function(z) { df(Fs,df1,df2,ncp=z,log=TRUE) }
 
@@ -571,7 +625,7 @@ predint <- function(x,oosdf,oosrescal=1/sqrt(oosdf+1),ope=NULL,level=0.95,
 	}
 	ncp.MLE <- optimize(max.func,c(lb,ub),maximum=TRUE)$maximum;
 	return(ncp.MLE)
-} # nocov end
+}
 .F_ncp_MLE <- Vectorize(.F_ncp_MLE_single,
 											vectorize.args = c("Fs","df1","df2"),
 											SIMPLIFY = TRUE)
@@ -661,7 +715,7 @@ T2.inference <- function(T2,df1,df2,...) {
 #' @references 
 #'
 #' Kubokawa, T., C. P. Robert, and A. K. Saleh. "Estimation of noncentrality parameters." 
-#' Canadian Journal of Statistics 21, no. 1 (1993): 45-57. \url{http://www.jstor.org/stable/3315657}
+#' Canadian Journal of Statistics 21, no. 1 (1993): 45-57. \url{https://www.jstor.org/stable/3315657}
 #'
 #' Spruill, M. C. "Computation of the maximum likelihood estimate of a noncentrality parameter." 
 #' Journal of multivariate analysis 18, no. 2 (1986): 216-224.
@@ -720,7 +774,7 @@ inference <- function(z.s,type=c("KRS","MLE","unbiased")) {
 }
 #' @rdname inference
 #' @method inference sropt
-#' @S3method inference sropt
+#' @export
 inference.sropt <- function(z.s,type=c("KRS","MLE","unbiased")) {
 	# type defaults to "KRS":
 	type <- match.arg(type)
@@ -732,7 +786,7 @@ inference.sropt <- function(z.s,type=c("KRS","MLE","unbiased")) {
 }
 #' @rdname inference
 #' @method inference del_sropt
-#' @S3method inference del_sropt
+#' @export
 inference.del_sropt <- function(z.s,type=c("KRS","MLE","unbiased")) {
 	# type defaults to "KRS":
 	type <- match.arg(type)
